"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = buildCss;

var _path = _interopRequireDefault(require("path"));

var _gulp = require("gulp");

var _gulpPlumber = _interopRequireDefault(require("gulp-plumber"));

var _gulpIf = _interopRequireDefault(require("gulp-if"));

var _gulpStylus = _interopRequireDefault(require("gulp-stylus"));

var _gulpPostcss = _interopRequireDefault(require("gulp-postcss"));

var _autoprefixer = _interopRequireDefault(require("autoprefixer"));

var _cssMqpacker = _interopRequireDefault(require("css-mqpacker"));

var _postcssUncss = _interopRequireDefault(require("postcss-uncss"));

var _postcssCsso = _interopRequireDefault(require("postcss-csso"));

var _postcssUrl = _interopRequireDefault(require("postcss-url"));

var _gulpHeader = _interopRequireDefault(require("gulp-header"));

var _gulpRename = _interopRequireDefault(require("gulp-rename"));

var _gulpGzip = _interopRequireDefault(require("gulp-gzip"));

var _fancyLog = _interopRequireDefault(require("fancy-log"));

var _gulpUtilErrorHandler = _interopRequireDefault(require("@hidoo/gulp-util-error-handler"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

let pkg = {}; // try to load package.json that on current working directory

try {
  pkg = require(_path.default.resolve(process.cwd(), 'package.json')); // eslint-disable-line global-require
} catch (error) {
  _fancyLog.default.error('Failed to load package.json.');
}
/**
 * task default options.
 * @type {Object}
 */


const DEFAULT_OPTIONS = {
  name: 'build:css',
  src: null,
  dest: null,
  filename: 'main.css',
  suffix: '.min',
  browsers: ['> 0.5% in JP', 'ie >= 10', 'android >= 4.4'],
  banner: '',
  stylusOptions: {
    rawDefine: {}
  },
  url: null,
  urlOptions: {},
  uncssTargets: [],
  uncssIgnore: [],
  additionalProcess: null,
  compress: false
};
/**
 * return css build task by stylus
 * @param  {Object} options - options
 * @param  {String} [options.name='build:css'] - task name (use as displayName)
 * @param  {String} options.src - source path
 * @param  {String} options.dest - destination path
 * @param  {String} [options.filename='main.css'] - destination filename
 * @param  {String} [options.suffix='.min'] - suffix when compressed
 * @param  {Array<String>} [options.browsers] - target browsers.
 *   see: {@link http://browserl.ist/?q=%3E+0.5%25+in+JP%2C+ie%3E%3D+10%2C+android+%3E%3D+4.4 default target browsers}
 * @param  {String} [options.banner=''] - license comments
 * @param  {Object} [options.stylusOptions={rawDefine: {}}] - stylus options.
 *   see: {@link https://www.npmjs.com/package/gulp-stylus gulp-stylus options}
 * @param  {String} [options.url=null] - type of processing of url() (one of [inline|copy|rebase])
 *   see: {@link https://www.npmjs.com/package/postcss-url}
 * @param  {Object} [options.urlOptions={}] - options of processing of url()
 *   see: {@link https://www.npmjs.com/package/postcss-url#options-combinations}
 * @param  {Array<String>} [options.uncssTargets=[]] - array of html file path that target of uncss process
 * @param  {Array<String>} [options.uncssIgnore=[]] - array of selector that should not removed
 * @param  {Function<PostCSSRoot>} [options.additionalProcess=null] - additional PostCss process
 * @param  {Boolean} [options.compress=false] - compress file or not
 * @return {Function<Stream>}
 *
 * @example
 * import {task} from 'gulp';
 * import buildCss from '@hidoo/gulp-task-build-css-stylus';
 *
 * task('css', buildCss({
 *   name: 'css:main',
 *   src: '/path/to/stylus/main.styl',
 *   dest: '/path/to/dest'
 *   filename: 'styles.css',
 *   suffix: '.hoge',
 *   browsers: ['> 0.1% in JP'],
 *   banner: '/*! copyright <%= pkg.author %> *\/\n', // end of comment is not need to escape actually.
 *   stylusOptions: {rawDefine: {}},
 *   url: 'inline',
 *   urlOptions: {basePath: path.resolve(process.cwd(), 'src/images'},
 *   uncssTargets: ['/path/to/html/target.html'],
 *   uncssIgnore: ['.ignore-selector'],
 *   additionalProcess: (root) => root,
 *   compress: true
 * }));
 */

function buildCss(options = {}) {
  const opts = _objectSpread({}, DEFAULT_OPTIONS, {}, options); // define task


  const task = () => {
    const {
      filename,
      suffix,
      browsers,
      banner,
      uncssTargets,
      uncssIgnore,
      additionalProcess,
      compress
    } = opts,
          extname = _path.default.extname(filename),
          basename = _path.default.basename(filename, extname),
          stylusOptions = opts.stylusOptions || {},
          postProcesses = []; // additinal stylus data
    // + always add NODE_ENV


    const mergedStylusOptions = _objectSpread({}, stylusOptions, {
      rawDefine: _objectSpread({}, stylusOptions.rawDefine, {
        NODE_ENV: process.env.NODE_ENV || 'development' // eslint-disable-line no-process-env

      })
    }); // add default post css process


    postProcesses.push((0, _autoprefixer.default)({
      overrideBrowserslist: browsers
    }));
    postProcesses.push(_cssMqpacker.default); // add post css process by postcss-url

    if (typeof opts.url === 'string' && ['inline', 'copy', 'rebase'].includes(opts.url)) {
      postProcesses.push((0, _postcssUrl.default)(_objectSpread({}, opts.urlOptions, {
        url: opts.url
      })));
    } // add post css process if opts.uncssTargets is valid


    if (Array.isArray(uncssTargets) && Boolean(uncssTargets.length)) {
      postProcesses.push((0, _postcssUncss.default)({
        html: uncssTargets,
        ignore: Array.isArray(uncssIgnore) ? uncssIgnore : []
      }));
    } // add post css process if opts.additionalProcess is valid


    if (typeof additionalProcess === 'function') {
      postProcesses.push(additionalProcess);
    }

    return (0, _gulp.src)(opts.src).pipe((0, _gulpPlumber.default)({
      errorHandler: _gulpUtilErrorHandler.default
    })).pipe((0, _gulpStylus.default)(mergedStylusOptions)).pipe((0, _gulpPostcss.default)([...postProcesses])).pipe((0, _gulpHeader.default)(banner, {
      pkg
    })).pipe((0, _gulpRename.default)({
      basename,
      extname
    })).pipe((0, _gulp.dest)(opts.dest)).pipe((0, _gulpIf.default)(compress, (0, _gulpRename.default)({
      suffix
    }))).pipe((0, _gulpIf.default)(compress, (0, _gulpPostcss.default)([(0, _postcssCsso.default)({
      restructure: false
    })]))).pipe((0, _gulpIf.default)(compress, (0, _gulp.dest)(opts.dest))).pipe((0, _gulpIf.default)(compress, (0, _gulpGzip.default)({
      append: true
    }))).pipe((0, _gulpIf.default)(compress, (0, _gulp.dest)(opts.dest)));
  }; // add displayName (used as task name for gulp)


  if (typeof opts.name === 'string' && opts.name !== '') {
    task.displayName = opts.name;
  }

  return task;
}